@@ -1,5 +1,6 @@
 """SQL utility functions for escaping and formatting values."""
 
+import inspect
 import json
 from dataclasses import is_dataclass
 from datetime import date, datetime
@@ -143,6 +144,8 @@ def get_sql_type_string(py_type: Type, dialect: str) -> str:
     Returns:
         SQL type string
     """
+    from typing import get_args
+
     # Import here to avoid circular imports
     from ._types import is_struct_type
 
@@ -153,15 +156,57 @@ def get_sql_type_string(py_type: Type, dialect: str) -> str:
     if py_type in type_mapping:
         return type_mapping[py_type]
 
+    # Handle List types
+    if hasattr(py_type, "__origin__") and py_type.__origin__ is list:
+        element_type = get_args(py_type)[0] if get_args(py_type) else str
+        element_sql_type = get_sql_type_string(element_type, dialect)
+        if dialect == "bigquery":
+            return f"ARRAY<{element_sql_type}>"
+        else:
+            return f"ARRAY({element_sql_type})"
+
+    # Handle Dict/Map types
+    if hasattr(py_type, "__origin__") and py_type.__origin__ is dict:
+        type_args = get_args(py_type)
+        key_type = type_args[0] if type_args else str
+        value_type = type_args[1] if len(type_args) > 1 else str
+
+        if dialect == "bigquery":
+            # BigQuery stores dicts as JSON strings
+            return "STRING"
+        elif dialect in ("athena", "trino"):
+            # Athena/Trino use native MAP type
+            key_sql_type = get_sql_type_string(key_type, dialect)
+            value_sql_type = get_sql_type_string(value_type, dialect)
+            return f"MAP({key_sql_type}, {value_sql_type})"
+        elif dialect == "redshift":
+            # Redshift uses SUPER type for JSON
+            return "SUPER"
+        elif dialect == "snowflake":
+            # Snowflake uses VARIANT type for JSON
+            return "VARIANT"
+        else:
+            # Default to string for other dialects
+            return "VARCHAR"
+
     # Check if it's a struct type
     if is_struct_type(py_type):
-        # Build nested ROW type recursively
+        # Build nested struct type recursively
         nested_hints = get_type_hints(py_type)
         nested_fields = []
         for nested_name, nested_type in nested_hints.items():
             nested_sql = get_sql_type_string(nested_type, dialect)
-            nested_fields.append(f"{nested_name} {nested_sql}")
-        return f"ROW({', '.join(nested_fields)})"
+            if dialect == "bigquery":
+                # BigQuery uses STRUCT<field_name type> syntax
+                nested_fields.append(f"{nested_name} {nested_sql}")
+            else:
+                # Athena/Trino use ROW syntax
+                nested_fields.append(f"{nested_name} {nested_sql}")
+
+        if dialect == "bigquery":
+            return f"STRUCT<{', '.join(nested_fields)}>"
+        else:
+            return f"ROW({', '.join(nested_fields)})"
 
     # For other complex types, default to VARCHAR
     return type_mapping.get(str, "VARCHAR")
@@ -329,6 +374,13 @@ def format_sql_value(value: Any, column_type: Type, dialect: str = "standard") -
             for element in value:
                 formatted_element = format_sql_value(element, element_type, dialect)
                 formatted_elements.append(formatted_element)
+
+            # Handle empty arrays with explicit type casting
+            if not formatted_elements:
+                # Get the SQL type for the element
+                sql_element_type = get_sql_type_string(element_type, dialect)
+                return f"CAST([] AS ARRAY<{sql_element_type}>)"
+
             return f"[{', '.join(formatted_elements)}]"
         elif dialect in ("athena", "trino"):
             # Format each element in the array for Athena/Trino
@@ -418,14 +470,16 @@ def format_sql_value(value: Any, column_type: Type, dialect: str = "standard") -
         else:
             return escape_sql_string(str(value))
 
-    # Handle numeric types
-    elif column_type in (int, float):
-        return str(value)
-
-    # Handle boolean types
+    # Handle boolean types (must come before numeric since bool is subclass of int)
     elif column_type is bool:
         return "TRUE" if value else "FALSE"
 
+    # Handle numeric types
+    elif column_type in (int, float) or (
+        inspect.isclass(column_type) and issubclass(column_type, (int, float))
+    ):
+        return str(value)
+
     # Handle date types
     elif column_type is date:
         if dialect == "bigquery":
@@ -457,8 +511,14 @@ def format_sql_value(value: Any, column_type: Type, dialect: str = "standard") -
                 return f"TIMESTAMP '{value}'"
 
     # Handle decimal types
-    elif column_type == Decimal:
-        return str(value)
+    elif column_type == Decimal or (
+        inspect.isclass(column_type) and issubclass(column_type, Decimal)
+    ):
+        if dialect == "bigquery":
+            # BigQuery needs explicit NUMERIC casting for decimals
+            return f"NUMERIC '{value}'"
+        else:
+            return str(value)
 
     # Default: convert to string
     else:
@@ -510,6 +570,33 @@ def _format_struct_value(value: Any, struct_type: Type, dialect: str) -> str:
 
         return f"CAST(ROW({', '.join(field_values)}) AS {row_type})"
 
+    # For BigQuery
+    elif dialect == "bigquery":
+        # Handle NULL struct values
+        if value is None:
+            return "NULL"
+
+        # Get type hints for the struct
+        type_hints = get_type_hints(struct_type)
+
+        # Format non-NULL struct values as named structs
+        field_pairs = []
+        for field_name, field_type in type_hints.items():
+            # Get the field value
+            if is_dataclass(value):
+                field_value = getattr(value, field_name, None)
+            elif is_pydantic_model_class(type(value)):
+                field_value = getattr(value, field_name, None)
+            else:
+                # If it's a dict
+                field_value = value.get(field_name) if isinstance(value, dict) else None
+
+            # Format the field value recursively
+            formatted_value = format_sql_value(field_value, field_type, dialect)
+            field_pairs.append(f"{formatted_value} AS {field_name}")
+
+        return f"STRUCT({', '.join(field_pairs)})"
+
     # For other databases, struct support would need to be implemented
     else:
         raise NotImplementedError(f"Struct type not yet supported for dialect: {dialect}")
