@@ -3,7 +3,18 @@
 from dataclasses import is_dataclass
 from datetime import date, datetime
 from decimal import Decimal
-from typing import Any, Dict, List, Type, TypeVar, Union, cast, get_args, get_origin, get_type_hints
+from typing import (
+    Any,
+    Dict,
+    List,
+    Type,
+    TypeVar,
+    Union,
+    cast,
+    get_args,
+    get_origin,
+    get_type_hints,
+)
 
 
 T = TypeVar("T")
@@ -113,6 +124,8 @@ def _parse_string_value(value_str: str) -> Any:
         return None
 
     # Try to parse as number
+    # Note: Callers should check if the target type is str before calling this function
+    # to preserve numeric-looking strings with leading zeros (e.g., "02101" zip codes)
     try:
         if "." in value_str:
             return float(value_str)
@@ -350,7 +363,9 @@ class BaseTypeConverter:
 
                 # Check if it's a simple comma-separated format (like Athena returns)
                 # e.g., "{John Doe, 30, 75000.5, {123 Main St, New York, 10001}, true}"
-                if "=" not in inner_value or inner_value.count(",") > inner_value.count("="):
+                # We've now fixed the parser to handle mixed format with lists and maps:
+                # {key=value, list=[item1, item2], map={k1=v1, k2=v2}}
+                if "=" not in inner_value:
                     # This looks like positional values, not key=value pairs
                     field_names = _get_struct_field_names(target_type)
                     values = _parse_bracketed_string(value)
@@ -386,9 +401,24 @@ class BaseTypeConverter:
                         value_str = value_str.strip()
                         # Get the field type for this key
                         field_type = type_hints.get(key, str)
-                        # Parse and convert the value
-                        parsed_value = _parse_string_value(value_str)
-                        result[key] = self.convert(parsed_value, field_type)
+
+                        # Check if value_str is an array notation [item1, item2, ...]
+                        if value_str.startswith("[") and value_str.endswith("]"):
+                            # This is an array, convert it directly using the converter
+                            result[key] = self.convert(value_str, field_type)
+                        # Check if value_str is a map notation {k1=v1, k2=v2}
+                        elif value_str.startswith("{") and value_str.endswith("}"):
+                            # This is a map/struct, convert it directly
+                            result[key] = self.convert(value_str, field_type)
+                        else:
+                            # If the target type is string, preserve the value as-is
+                            # to maintain leading zeros in numeric-looking strings
+                            if field_type is str:
+                                result[key] = value_str
+                            else:
+                                # Parse and convert the value normally
+                                parsed_value = _parse_string_value(value_str)
+                                result[key] = self.convert(parsed_value, field_type)
 
                 return _create_struct_instance(target_type, result)
 
